# Generated by Django 1.9.5 on 2016-07-28 02:33

from django.db import migrations


class Migration(migrations.Migration):
    replaces = [("social_auth", "0005_auto_20160727_2333")]

    dependencies = [
        ("social_django", "0004_auto_20160423_0400"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="association",
            unique_together={("server_url", "handle")},
        ),
    ]
